package invertedpendulum;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class Canvas  extends JFrame {
	private static final long serialVersionUID = 1L;
	private BufferedImage canvas;
    int canvasW=512,canvasH=800;
    boolean simulate=true;
    
    Graph thetaG, omegaG, currentG;
    
    
    public Pendulum pendulum=new Pendulum();
    
    /** Creates a new instance of Canvas */
    public Canvas() {
	super("Inverted Pendulum");	//set caption of window
        this.setBounds(0,0,canvasW,canvasH);
        this.setResizable(false);
        
        addMouseMotionListener(new MyMouseMotionListener());        
        addWindowListener( new WindowAdapter(){
            //If this event is not handled, the application will
            //be running even after the window is closed.
            public void windowClosing(WindowEvent e){
                System.exit(0);
            }
        });        
        canvas=new BufferedImage(canvasW,canvasH,BufferedImage.TYPE_INT_RGB);
        clearImage(canvas);
        initGraphs();
        
        
        this.setVisible(true);
        run();
    }
    
    private void initGraphs(){
    	thetaG=new Graph(500);
    	omegaG=new Graph(500);
    	currentG=new Graph(500);
    	thetaG.setColor(Color.RED);
    	omegaG.setColor(Color.GREEN);
    	currentG.setColor(Color.BLUE);

    	thetaG.setScale(1,100);
    	omegaG.setScale(1,100);
    	currentG.setScale(1,30);    	
    	
    	thetaG.setOffset(0, 600);
    	omegaG.setOffset(0, 600);
    	currentG.setOffset(0,600);
    	   	
    	thetaG.setYBounds(-2, 2);
    	omegaG.setYBounds(-2,2);
    	currentG.setYBounds(-20, 20);
    }
    
    private void getGraphValues(){
    	thetaG.insertValue((float)pendulum.getTheta());
    	omegaG.insertValue((float)pendulum.getOmega());
    	currentG.insertValue((float)pendulum.getCurrent());    	
    }
    
    private void clearImage(BufferedImage c){
        Graphics temp=c.getGraphics();
        temp.setColor(Color.BLACK);
        temp.fillRect(0, 0, canvasW, canvasH);
        temp.setColor(Color.WHITE);
        temp.dispose();
    }
    
//This method is used to run a thread that keeps simulating and redrawing the screen.
    public void run(){
        while(true){
            try{
            	if (simulate){
            		pendulum.simulate(0.01);
            		getGraphValues();
            	}
                clearImage(canvas);
                paintMethod();
                Thread.sleep(20);//paint about 50 times a second
            }
            catch (Exception e){
                System.out.println(e.toString());
            }
        }
    }
    
    void paintMethod(){
        Graphics temp1=this.getGraphics();
        Graphics temp2=canvas.getGraphics();

        pendulum.draw(temp2,250,250);
        thetaG.draw(temp2);
        temp2.drawChars("Theta".toCharArray(), 0, 5, 0, 720);
        omegaG.draw(temp2);
        temp2.drawChars("Omega".toCharArray(), 0, 5, 0, 740);
        currentG.draw(temp2);
        temp2.drawChars("Current".toCharArray(), 0, 7, 0, 760);
        temp2.setColor(Color.WHITE);

        //This draws the contents of the hidden canvas onto the visible JFrame		
        temp1.drawImage(canvas, this.getInsets().left, this.getInsets().top, canvasW, canvasH, this);       

        temp1.dispose();
        temp2.dispose();
    }

    public class MyMouseMotionListener implements MouseMotionListener{
        public void mouseMoved(MouseEvent me) {
        }
        public void mouseDragged(MouseEvent me) {
        	simulate=false;
            int xpos=me.getX()-250;
            int ypos=250-me.getY();
            double theta = Math.atan2(ypos,xpos);
            pendulum.setTheta(Math.PI/2-theta);
            simulate=true;
        }
    }
    
}
