
package invertedpendulum;
import java.awt.Graphics;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import net.sourceforge.jFuzzyLogic.FIS;
//import net.sourceforge.jFuzzyLogic.rule.*;

public class Pendulum {       
    private double theta;
    private double omega;

    private double radius;//radius of pendulum
    private double mass;//mass of pendulum blob
    private double gravity;//gravity of system

    private double motorK;//motor parameters

    private double motorI;//motor current

    private FIS fis;//fuzzy inference system

    public void simulate(double timeStep){
        double dOmega=0, motorTorque=0, momentInertia, massTorque;
        motorI=fuzzyCurrent();
        motorTorque     =  motorI*motorK;
        momentInertia   =  mass*radius*radius;
        massTorque      =  radius*mass*gravity*Math.sin(theta);
        //System.out.println("Current: " + motorI + ", torque: " + motorTorque + ", torque: "+ massTorque);
        dOmega= (massTorque + motorTorque)/momentInertia;
        omega = omega + dOmega * timeStep;
        theta = theta + omega * timeStep;
        
        //constrain the value of theta between -Pi and +Pi
        if (theta>Math.PI){
        	theta=theta-2*Math.PI;
        }else if (theta < -Math.PI){
        	theta=2*Math.PI + theta;
        }
        //System.out.println("Theta: "+theta + " Omega: "+omega);
    }
    
    /** Creates a new instance of Pendulum */
    public Pendulum() {
        initFuzzySystem();
        try {        
        	BufferedReader bf=new BufferedReader(new FileReader("config.txt"));        
			mass = Double.parseDouble(bf.readLine().split("\\s+")[1]);
			motorK = Double.parseDouble(bf.readLine().split("\\s+")[1]);
	        omega = Double.parseDouble(bf.readLine().split("\\s+")[1]);
	        radius = Double.parseDouble(bf.readLine().split("\\s+")[1]);
	        theta = Double.parseDouble(bf.readLine().split("\\s+")[1]);
	        gravity=Double.parseDouble(bf.readLine().split("\\s+")[1]);
        } catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    public void setTheta(double theta_value){
    	theta=theta_value;
    	omega=0;
    }
    
    public double getTheta(){
    	return theta;    	
    }
    
    public double getOmega(){
    	return omega;
    }
    
    public double getCurrent(){
    	return motorI;
    }
    
    void draw(Graphics g, int X, int Y){
        double radiusScale=100;
        int endx = X + (int)((double)radius*radiusScale * Math.sin(theta));
        int endy = Y - (int)((double)radius*radiusScale * Math.cos(theta));
        g.drawLine (X, Y,endx, endy);
        g.fillOval (endx-5, endy-5, 10,10);
        g.drawLine (X, Y, X - 10, Y + 10);
        g.drawLine (X, Y, X + 10, Y + 10);
        g.drawLine (X - 10, Y + 10, X + 10, Y + 10);
    }
    
    void initFuzzySystem(){
        // Load from 'FCL' file        
        String fileName = "pendulum.fcl";
        fis = FIS.load(fileName,true);
        // Error while loading?
        if( fis == null ) { 
            System.err.println("Can't load file: '" 
                                   + fileName + "'");
            return;
        }
        System.out.println("Loaded File");
        // Show   
        fis.chart();
    }
    
    double fuzzyCurrent(){
        // Set inputs
        fis.setVariable("theta", theta);
        fis.setVariable("omega", omega);
        // Evaluate
        fis.evaluate();
        // Show output variable's chart 
        //fis.getVariable("current").chartDefuzzifier(true);

        // Print ruleSet
        return fis.getVariable("current").defuzzify();
    }    
}
